#!/bin/sh
# file : update.sh
# features : script which handle the update on the board
# date : 06/02/2018
# author : aurelien BOUIN

#################################### Constants Definitions #################################

VERSION="2.0.19"

LOG_DURING_UPDATE="/tmp/file_during_update.txt"
LOG_UPDATE="/tmp/file_after_update.txt"
LOG_UPDATE_IN_DATAS="/datas/file_after_update$(date +%Y%m%d_%Hh%M).txt"
UPDATE_PIPE=""
TYPE_POSTE_PATH="/usr/etc/type_poste.conf"
STATIC_EXE_PLACE="/tmp/update/static-exe/"

BOOTLOADER_FLASH_PART="/dev/mtd/0"
BOOTLOADER_ENV_FLASH_PART="/dev/mtd/1"
LINUX_FLASH_PART="/dev/mtd/2"
DATAS_FLASH_PART="/dev/mtd/3"
ROOTFS_FLASH_PART="/dev/mtd/4"

#################################### Functions Declarations #################################

check_is_something_bad_in_dmesg() {
	TEST=$(dmesg |grep "page allocation failure")
	if [ 'ri'"$TEST"'en' != "rien" ]
	then
		printit "There was a page allocation failure before we will stop the upgrade process check with the technical support !!"
		printit "Il y a eu un problme de type page allocation failure sur le poste pendant la MAJ, verifier avec le support technique !!"
		printit "Ask reboot"
		$STATIC_EXE_PLACE/busybox reboot
	fi
	TEST=$(dmesg |grep "Kernel panic")
	if [ 'ri'"$TEST"'en' != "rien" ]
	then
		printit "There was a page allocation failure before we will stop the upgrade process check with the technical support !!"
		printit "Il y a eu un problme de type page allocation failure sur le poste pendant la MAJ, verifier avec le support technique !!"
		printit "Ask reboot"
		$STATIC_EXE_PLACE/busybox reboot
	fi
}
save_binary_into_ram() {
	mkdir -p /tmp/update/
	cp /bin/busybox /tmp/update/
	cp /usr/bin/flash* /tmp/update/
	cp /lib/ld-linux.so.3 /tmp/update/
	cp /lib/libc.so.6 /tmp/update/
	cp /lib/libgcc_s.so.1 /tmp/update/
	export LD_LIBRARY_PATH="/tmp/update/"
}
printit() {
	$STATIC_EXE_PLACE/busybox echo "$1";
	$STATIC_EXE_PLACE/busybox echo "$1" >> $LOG_DURING_UPDATE;
	$STATIC_EXE_PLACE/busybox echo "$1" >> $LOG_UPDATE;
	$STATIC_EXE_PLACE/busybox echo "$1" >> $LOG_UPDATE_IN_DATAS;
}
show_pourcent_to_ihm() {
	if [ ! 'ri'$UPDATE_PIPE'en' = 'rien' ]
	then
		$STATIC_EXE_PLACE/busybox echo "PERCENT_VALUE $1#" > $UPDATE_PIPE
	fi
}
show_first_info_to_ihm() {
	if [ ! 'ri'$UPDATE_PIPE'en' = 'rien' ]
	then
		$STATIC_EXE_PLACE/busybox echo "INFO1 $1#" > $UPDATE_PIPE
	fi
}
show_second_info_to_ihm() {
	if [ ! 'ri'$UPDATE_PIPE'en' = 'rien' ]
	then
		$STATIC_EXE_PLACE/busybox echo "INFO2 $1#" > $UPDATE_PIPE
	fi
}
ask_reboot() {
	printit "Ask reboot"
	$STATIC_EXE_PLACE/busybox reboot
}
ask_hard_reboot() {
	printit "Ask hard reboot"
	$STATIC_EXE_PLACE/busybox  reboot
}
ask_very_hard_reboot() {
	printit "Ask very hard reboot"
	#because missing -f in gip mounted when deleting /usb_datas
	$STATIC_EXE_PLACE/busybox umount /usb_datas
	$STATIC_EXE_PLACE/busybox rm -rf /usb_datas
	$STATIC_EXE_PLACE/busybox touch /usb_datas
	$STATIC_EXE_PLACE/busybox reboot
}
softsgipstop_light() {
	. $TYPE_POSTE_PATH
	if [ 'ri'"$ASTERISK_SERVER"'en' = 'riyen' ]
	then
	    . /etc/rc.d/init.d/asteriskd stop;
	    . /etc/rc.d/init.d/webak stop;
	    . /etc/rc.d/init.d/servmgrd stop;
	    . /etc/rc.d/init.d/snmpdd stop;
	else
	    . /etc/rc.d/init.d/goped stop;
	    $STATIC_EXE_PLACE/busybox sleep 1;
	    if [ -e /etc/rc.d/init.d/mediagipd ]
	    then
	        . /etc/rc.d/init.d/mediagipd stop;
	    else
	        $STATIC_EXE_PLACE/busybox echo "Do not stop mediagip because the init does not exist"
	    fi
	    if [ -e /etc/rc.d/init.d/vpuhandlerd ]
	    then
	        . /etc/rc.d/init.d/vpuhandlerd stop;
	        #just in case...
	        killall -9 vpuhandler
	        killall -9 vpuhandler_supervisor
	    else
	        $STATIC_EXE_PLACE/busybox echo "Do not stop vpuhandler because the init does not exist"
	    fi
	    . /etc/rc.d/init.d/displayappd stop;
	    . /etc/rc.d/init.d/webgip stop;
	    . /etc/rc.d/init.d/aecappd stop;
	    . /etc/rc.d/init.d/showgipd stop;
        . /etc/rc.d/init.d/notifierd stop;
	    #just to be sure
	    $STATIC_EXE_PLACE/busybox killall -9 showgip
	    . /etc/rc.d/init.d/snmpagentd stop;
	    . /etc/rc.d/init.d/snmpdd stop;
	    . /etc/rc.d/init.d/keepfbbusy stop;
	    $STATIC_EXE_PLACE/busybox killall -9 mediagip_supervisor
	    $STATIC_EXE_PLACE/busybox killall -9 mediagip
	fi	
}
check_datas_partition(){
	IS_DATAS_MOUNT=$(df |grep "datas")
	if [ 'ri'"$IS_DATAS_MOUNT"'en' = "rien" ]
	then
		DATAS_PATH=/datas
		echo "For unknown reason /datas is not mounted"
		echo "mounting user datas from flash : $DATAS_PATH"
		rm -rf $DATAS_PATH
	  	mkdir $DATAS_PATH
	  	mount -t jffs2 /dev/mtdblock3 $DATAS_PATH
	  	if [ $? != 0 ]
	  	then
	  		echo "Unable to mount datas for unknown reason..."
	  		echo "We are going to erase it ..."
	  		flash_erase -u $DATAS_FLASH_PART 0 0
	  		mount -t jffs2 /dev/mtdblock3 $DATAS_PATH
	  		if [ $? != 0 ]
	  		then
	  			#do something here flash_unlock flash_erase ...
				echo "There is a really big difficulty with $DATAS_PATH"
				echo "We won't use it for update"
				LOG_UPDATE_IN_DATAS=/dev/null
			fi
	  	fi
	fi
}
stop_application_to_remount_root_as_readonly(){
	
#not used yet
MY_OWN_PID=$$
LIST_PID_USING_ROOT=$(fuser /)
for i in $LIST_PID_USING_ROOT
do
	if [ $MY_OWN_PID -ne $i ]
	then
		kill -9 $i
	fi
done
mount -o remount -r /dev/root /
$STATIC_EXE_PLACE/busybox watchdog -t 1 /dev/watchdog &
$STATIC_EXE_PLACE/busybox watchdog -t 1 /dev/watchdog &
$STATIC_EXE_PLACE/busybox watchdog -t 1 /dev/watchdog &
MOUNT_LOG=$(mount)
printit "Mount : "
printit "$MOUNT_LOG"

}
stop_maximum_applications(){
	#Prevents applications to reboot the board (not the ugliest hack)
	#discovered in 2.12.5 with mope.
	rm /sbin/reboot
	#fake it
	echo -e "#!/bin/sh\nexit 0\n" > /sbin/reboot
	chmod +x /sbin/reboot

	printit "We are going to stop Network services you may not be able to spy what happen on board anymore ..."
	set -x
	printit "Stop services not needed :"
	printit "-displayapp :"
	/etc/rc.d/init.d/displayappd stop
	printit "-webgip :"
	/etc/rc.d/init.d/webgip stop
	printit "-snmpd :"
	/etc/rc.d/init.d/snmpdd stop
	printit "-snmpagent :"
	/etc/rc.d/init.d/snmpagentd stop
	printit "-show* :"
	/etc/rc.d/init.d/showgipd stop
	printit "-aecapp :"
	/etc/rc.d/init.d/aecappd stop
	printit "-notifier :"
    /etc/rc.d/init.d/notifierd stop
#printit "-network applications :"
#killall -q ethernet_ping.sh
#killall -q udhcpc
#printit "-sshd :"
#/etc/rc.d/init.d/sshd stop
#printit "-avahi :"
#/etc/rc.d/init.d/S50avahi-daemon stop
	printit "-ram_checker :"
	$STATIC_EXE_PLACE/busybox killall ram_checker.sh
	printit "-detect_rst_pressed :"
	$STATIC_EXE_PLACE/busybox ps | $STATIC_EXE_PLACE/busybox grep "detect_rst_pressed" | $STATIC_EXE_PLACE/busybox cut -d " " -f2 |$STATIC_EXE_PLACE/busybox xargs kill
	$STATIC_EXE_PLACE/busybox ps | $STATIC_EXE_PLACE/busybox grep "cat /dev/gip_io" | $STATIC_EXE_PLACE/busybox cut -d " " -f2 |$STATIC_EXE_PLACE/busybox xargs kill
#printit "-udevd :"
#killall udevd
	
	if [ -e /sbin/watchdog ]
	then
		printit "-gope :"
		/etc/rc.d/init.d/goped stop
		printit "-mediagip :"
		/etc/rc.d/init.d/mediagipd stop
		if [ -e /etc/rc.d/init.d/vpuhandlerd ]
	    then
	        /etc/rc.d/init.d/vpuhandlerd stop;
	        #just in case...
	        killall -9 vpuhandler
	        killall -9 vpuhandler_supervisor
	    else
	        printit "The system do not have vpuhandler"
	    fi
		printit "If the system reboot after this message, you will have to retry the upgrade"
		#2018/02/06 =>Just to be sure of watchdog realease
		kill -9 $(fuser /dev/watchdog)
		TRY_SOFTSGIPSTOP=0
		RUN_WHILE=1
		COUNT=0
		while [ $RUN_WHILE -eq 1 ]
		do
			PS_LINE=$( dmesg | grep Unexpected )
			if [ 'ri'"$PS_LINE"'en' != "rien" ]
			then
				RUN_WHILE=0
			else
				COUNT=$((COUNT + 1))
				if [ $COUNT -ge 20 ]
				then
					if [ $TRY_SOFTSGIPSTOP -eq 1 ]
					then
						RUN_WHILE=0
						printit "The watchdog end signal has not been caught : After softsgip stopped"
						printit "Going to do a ps to check later what is going on"
						printit "----------------------------------------------------------------------"
						$STATIC_EXE_PLACE/busybox  ps >> $LOG_UPDATE_IN_DATAS
						printit "----------------------------------------------------------------------"
						printit "Unable to update"
						ask_hard_reboot
					else
						COUNT=0
						RUN_WHILE=1
						TRY_SOFTSGIPSTOP=1
						#sometimes mediagip do not stop and it will make gope keep the watchdog
						#part of /etc/rc.d/init.d/softsgip stop
						printit "The watchdog end signal has not been caught : Trying to stop the softwares directly..."
						softsgipstop_light
						$STATIC_EXE_PLACE/busybox sleep 5
					fi
				else
					#the watchdog have not been caught yet
					$STATIC_EXE_PLACE/busybox sleep 1
				fi
			fi
		done
		printit "Starting busybox watchdog to keep the watchdog alive"
		$STATIC_EXE_PLACE/busybox watchdog -t 1 /dev/watchdog &
		$STATIC_EXE_PLACE/busybox ps
		
	#checking if the watchdog is kept
		if [ 1 -eq 1 ]
		then
		RUN_WHILE=1
		COUNT=0
		while [ $RUN_WHILE -eq 1 ]
		do
			PS_LINE=$(ps | grep watchdog | grep busybox | grep -v grep )
			if [ 'ri'"$PS_LINE"'en' != "rien" ]
			then
				COUNT=$((COUNT + 1))
				if [ $COUNT -ge 5 ]
				then
					printit "Ok the watchdog have been caught"
					RUN_WHILE=0
				fi
				$STATIC_EXE_PLACE/busybox sleep 1
			else
				printit "Unable to catch the watchdog : Ask Reboot"
				ask_hard_reboot
			fi
		done
		fi
		printit "Umount root partition"
		$STATIC_EXE_PLACE/busybox umount /
	else
		printit "There is no way to keep the watchdog alive so we won't stop gope while upgrading"
	fi	
}
check_executables_in_place() {
	#sometimes (TFTP mode) the executables are all in the same directory
	[ ! -e $STATIC_EXE_PLACE/flash_unlock ] && STATIC_EXE_PLACE="/tmp/update/"
	
	for i in flash_unlock flash_erase flashcp busybox
	do
		if [ ! -e $STATIC_EXE_PLACE/$i ]
		then
			echo "$i is missing we won't be able to update... something went wrong retry it ..."
			ask_hard_reboot
			exit
		else
			chmod 777 $STATIC_EXE_PLACE/$i
		fi
	done
	
	printit "All the executables needed are ready to be used"
}

############################################ MAIN ############################################ 

echo "Firmware Update $VERSION"
POURCENT=0

rm -f $LOG_DURING_UPDATE $LOG_UPDATE /datas/file_after_update.txt

check_datas_partition

touch $LOG_UPDATE_IN_DATAS
if [ $? != 0 ]
then
	echo "There is a problem with the creation of the file $LOG_UPDATE_IN_DATAS => using /dev/null instead"
	LOG_UPDATE_IN_DATAS=/dev/null
fi

#save_binary_into_ram
check_executables_in_place

printit "################################ On board results ################################"
printit "Starting update v:$VERSION at $(date +%H:%M:%S) the $(date +%d/%m/%Y )"

check_is_something_bad_in_dmesg

$STATIC_EXE_PLACE/busybox ln -s $LOG_UPDATE_IN_DATAS /datas/file_after_update.txt

POURCENT=1
printit "Checking the md5sum of the files received on the target"

MD5_LIST=$(ls *.md5 2> /dev/null)
for i in $MD5_LIST
do
	if ! $STATIC_EXE_PLACE/busybox md5sum -c $i
	then
		[ 'ri'"$i"'en' = "riOK_FOR_DOWNGRADE.md5en" ] && continue
		printit "The file $i has not been received on the target correctly, will reboot"
		ask_reboot
		exit
	fi
done
POURCENT=2

printit "Checking if we are not downgrading"

show_pourcent_to_ihm "$POURCENT"
show_first_info_to_ihm "Verification prealable"
show_second_info_to_ihm ""

if [ ! -e /usr/etc/GIP_VERSION ]
then
	printit "We could not find /usr/etc/GIP_VERSION seems like we are prototyping... ok to continue..."
else
	if [ ! -e /tmp/update/UPDATE_VERSION ]
	then
		printit "We could not find /tmp/update/UPDATE_VERSION seems like we are prototyping... ok to continue..."
	else
		VERSION_TO_UPDATE_TIME_GET=$(cat /tmp/update/UPDATE_VERSION |grep time | cut -d':' -f2)
		VERSION_TO_UPDATE_GET=$(cat /tmp/update/UPDATE_VERSION |grep version | cut -d':' -f2)
		VERSION_TO_UPDATE_UNIT=$(echo $VERSION_TO_UPDATE_GET |cut -d'.' -f3)
		VERSION_TO_UPDATE_MILLE=$(echo $VERSION_TO_UPDATE_GET |cut -d'.' -f2)
		VERSION_TO_UPDATE_MILLION=$(echo $VERSION_TO_UPDATE_GET |cut -d'.' -f1)
		VERSION_TO_UPDATE=$((($VERSION_TO_UPDATE_MILLION * 1000000) + ($VERSION_TO_UPDATE_MILLE * 1000) + ($VERSION_TO_UPDATE_UNIT )))
		
		VERSION_FROM_UPDATE_TIME_GET=$(cat /usr/etc/GIP_VERSION |grep time | cut -d':' -f2)
		VERSION_FROM_UPDATE_GET=$(cat /usr/etc/GIP_VERSION |grep version | cut -d':' -f2)
		VERSION_FROM_UPDATE_UNIT=$(echo $VERSION_FROM_UPDATE_GET |cut -d'.' -f3)
		VERSION_FROM_UPDATE_MILLE=$(echo $VERSION_FROM_UPDATE_GET |cut -d'.' -f2)
		VERSION_FROM_UPDATE_MILLION=$(echo $VERSION_FROM_UPDATE_GET |cut -d'.' -f1)
		VERSION_FROM_UPDATE=$((($VERSION_FROM_UPDATE_MILLION * 1000000) + ($VERSION_FROM_UPDATE_MILLE * 1000) + ($VERSION_FROM_UPDATE_UNIT )))
		
		if [ ! 'ri'$UPDATE_PIPE'en' = 'rien' ]
		then
			$STATIC_EXE_PLACE/busybox echo "VERSION1 Ancienne version: $VERSION_FROM_UPDATE_GET#" > $UPDATE_PIPE
			$STATIC_EXE_PLACE/busybox echo "VERSION2 Nouvelle version: $VERSION_TO_UPDATE_GET#" > $UPDATE_PIPE
		fi
		
		if test $VERSION_FROM_UPDATE -gt $VERSION_TO_UPDATE 
		then
			if [ -e /tmp/update/OK_FOR_DOWNGRADE -a $(cat /tmp/update/OK_FOR_DOWNGRADE) -eq 1 ]
			then
				printit "Apparently you are aware that you are downgrading"
			else
				printit "We cannot downgrade... we are going to stop the process"
				POURCENT=100
				ask_reboot
				exit
			fi
		fi
		printit "Update version $VERSION_TO_UPDATE_GET:$VERSION_FROM_UPDATE_TIME_GET =>  $VERSION_FROM_UPDATE_GET:$VERSION_TO_UPDATE_TIME_GET"
	fi
fi

POURCENT=3
printit "-----------------------------------------------------------------------------------------------------------------------------"
printit "         Showing informations about the system before update :         " 
printit "-----------------------------------------------------------------------------------------------------------------------------"
UPTIME_LOG=$(uptime)
printit "Uptime : $UPTIME_LOG"
FREE_LOG=$(free)
printit "Free : "
printit "$FREE_LOG"
PARTITIONS_LOG=$(cat /proc/partitions)
printit "/proc/partitions : "
printit "$PARTITIONS_LOG"
PS_LOG=$(ps)
printit "ps : "
printit "$PS_LOG"
TOP_LOG=$(top -n 1)
printit "top : "
printit "$TOP_LOG"
DF_LOG=$(df)
printit "df : "
printit "$DF_LOG"
##### TOO much log with the time
#CHECK_OLD_UPDATE_LOG=$(grep "version" /datas/file_after_update* |grep -v file_after_update.txt)
#printit "Older update : "
#printit "$CHECK_OLD_UPDATE_LOG"

DMESG_LOG=$(dmesg | grep -v kbd_cypress |grep -v keycode )
printit "dmesg : "
printit "$DMESG_LOG"

printit "-----------------------------------------------------------------------------------------------------------------------------"

stop_maximum_applications
#stop_application_to_remount_root_as_readonly

POURCENT=4

printit "Copying busybox and flashcp"
show_pourcent_to_ihm "$POURCENT"
show_first_info_to_ihm "Copie de busybox"
show_second_info_to_ihm ""

POURCENT=5

printit "Showing the update screen"
export TSLIB_TSDEVICE=/dev/input/tslib0
export TSLIB_CONSOLEDEVICE=none
export LD_PRELOAD="/usr/lib/libts.so"

. /usr/etc/type_poste.conf

if [ 'y'"$TYPE_ECRAN"'n' = "yTFTn" ]
then
	if [ -e /usr/etc/tftupdate ]
	then
		UPDATE_PIPE="/tmp/updateInputPipe"
		$STATIC_EXE_PLACE/busybox rm -f $UPDATE_PIPE
		
		cd /usr/etc/tftupdate
		tftupdate -with-progression &
		cd -
	fi	
fi
if [ 'y'"$TYPE_ECRAN"'n' = "yLCDn" ]
then
	if [ -e /usr/etc/graphupdate ]
	then
		UPDATE_PIPE="/tmp/updateInputPipe"
		$STATIC_EXE_PLACE/busybox rm -f $UPDATE_PIPE
		
		cd /usr/etc/graphupdate
		graphupdate & 
		cd -
	fi	
fi

if [ ! 'ri'"$UPDATE_PIPE"'en' = 'rien' ]
then
	printit "Starting update application..."
	
	UPDATE_PIPE_COUNTDOWN=10
	while [ ! -p $UPDATE_PIPE -a $UPDATE_PIPE_COUNTDOWN -ne 0 ]
	do
		$STATIC_EXE_PLACE/busybox sleep 1
		UPDATE_PIPE_COUNTDOWN=`expr $UPDATE_PIPE_COUNTDOWN - 1`
	done
	
	if [ $UPDATE_PIPE_COUNTDOWN -eq 0 ]
	then
		printit "Could not send update informations !"
		UPDATE_PIPE=""
	fi
fi

POURCENT=6

PS_LOG=$(ps)
printit "ps after stopping almost everything : "
printit "$PS_LOG"

POURCENT=7

printit "Updating Partitions at $(date +%H:%M:%S) the $(date +%d/%m/%Y )"
cd /

BOOTLOADER_FILE=""
[ -e /tmp/update/barebox.bin ] && BOOTLOADER_FILE=/tmp/update/barebox.bin
[ -e /tmp/update/uboot.bin ] && BOOTLOADER_FILE=/tmp/update/uboot.bin
if [ -n $BOOTLOADER_FILE ]
then
	printit "Flashing the bootloader at $(date +%H:%M:%S) the $(date +%d/%m/%Y )"
	
	$STATIC_EXE_PLACE/busybox echo "Erasing the bootloader partition, that could take time, be patient !" >> $LOG_DURING_UPDATE
	show_pourcent_to_ihm "$POURCENT"
	show_second_info_to_ihm "Etape 1/2"
	$STATIC_EXE_PLACE/flash_erase -u $BOOTLOADER_FLASH_PART 0 0 >> $LOG_UPDATE_IN_DATAS
	POURCENT=12
		
	$STATIC_EXE_PLACE/busybox echo "Copying the bootloader partition, that could take time, be patient !" >> $LOG_DURING_UPDATE
	show_pourcent_to_ihm "$POURCENT"
	show_second_info_to_ihm "Etape 2/2"
	$STATIC_EXE_PLACE/flashcp -v $BOOTLOADER_FILE $BOOTLOADER_FLASH_PART >> $LOG_UPDATE_IN_DATAS
	if [ $? != 0 ]
	then
		printit "There was a problem during flash ... we are going to retry one time"
		$STATIC_EXE_PLACE/flash_unlock $BOOTLOADER_FLASH_PART >> $LOG_UPDATE_IN_DATAS
		$STATIC_EXE_PLACE/flash_erase $BOOTLOADER_FLASH_PART 0 0 >> $LOG_UPDATE_IN_DATAS
		$STATIC_EXE_PLACE/flashcp -v $BOOTLOADER_FILE $BOOTLOADER_FLASH_PART >> $LOG_UPDATE_IN_DATAS
		if [ $? != 0 ]
		then
			printit "There was a problem during flash for the second time ... we are going to stop update and not reboot ..."
			ask_very_hard_reboot
			exit
		fi
	fi
fi
POURCENT=20

if [ -e /tmp/update/uImage ]
then
	printit "Flashing the uimage at $(date +%H:%M:%S) the $(date +%d/%m/%Y )"
	
	$STATIC_EXE_PLACE/busybox echo "Erasing the Linux partition, that could take time, be patient !" >> $LOG_DURING_UPDATE
	show_pourcent_to_ihm "$POURCENT"
	show_second_info_to_ihm "Etape 1/2"
	$STATIC_EXE_PLACE/flash_erase -u $LINUX_FLASH_PART 0 0 >> $LOG_UPDATE_IN_DATAS
	POURCENT=25
		
	$STATIC_EXE_PLACE/busybox echo "Copying the Linux partition, that could take time, be patient !" >> $LOG_DURING_UPDATE
	show_pourcent_to_ihm "$POURCENT"
	show_second_info_to_ihm "Etape 2/2"
	$STATIC_EXE_PLACE/flashcp -v /tmp/update/uImage $LINUX_FLASH_PART >> $LOG_UPDATE_IN_DATAS
	if [ $? != 0 ]
	then
		printit "There was a problem during flash ... we are going to retry one time"
		$STATIC_EXE_PLACE/flash_unlock $LINUX_FLASH_PART >> $LOG_UPDATE_IN_DATAS
		$STATIC_EXE_PLACE/flash_erase $LINUX_FLASH_PART 0 0 >> $LOG_UPDATE_IN_DATAS
		$STATIC_EXE_PLACE/flashcp -v /tmp/update/uImage $LINUX_FLASH_PART >> $LOG_UPDATE_IN_DATAS
		if [ $? != 0 ]
		then
			printit "There was a problem during flash for the second time ... we are going to stop update and not reboot ..."
			ask_very_hard_reboot
			exit
		fi
	fi
fi
POURCENT=30

if [ -e /tmp/update/rootfs.jffs2 ]
then
	printit "Flashing the rootfs.jffs2 at $(date +%H:%M:%S) the $(date +%d/%m/%Y )"
		
	$STATIC_EXE_PLACE/busybox echo "Erasing the System partition, that could take time, be patient !" >> $LOG_DURING_UPDATE
	show_pourcent_to_ihm "$POURCENT"
	show_second_info_to_ihm "Etape 1/2"
	$STATIC_EXE_PLACE/flash_erase -u $ROOTFS_FLASH_PART 0 0 >> $LOG_UPDATE_IN_DATAS
	POURCENT=72
		
	$STATIC_EXE_PLACE/busybox echo "Copying the System partition, that could take time, be patient !" >> $LOG_DURING_UPDATE
	show_pourcent_to_ihm "$POURCENT"
	show_second_info_to_ihm "Etape 2/2"
	$STATIC_EXE_PLACE/flashcp -v /tmp/update/rootfs.jffs2 $ROOTFS_FLASH_PART >> $LOG_UPDATE_IN_DATAS
	if [ $? != 0 ]
	then
		printit "There was a problem during flash ... we are going to retry one time"
		$STATIC_EXE_PLACE/flash_unlock $ROOTFS_FLASH_PART >> $LOG_UPDATE_IN_DATAS
		$STATIC_EXE_PLACE/flash_erase $ROOTFS_FLASH_PART 0 0 >> $LOG_UPDATE_IN_DATAS
		$STATIC_EXE_PLACE/flashcp -v /tmp/update/rootfs.jffs2 $ROOTFS_FLASH_PART >> $LOG_UPDATE_IN_DATAS
		if [ $? != 0 ]
		then
			printit "There was a problem during flash for the second time ... we are going to stop update and not reboot ..."
			ask_very_hard_reboot
			exit
		fi
	fi
fi

POURCENT=88

if [ -e /tmp/update/things_to_execute.sh ]
then
	printit "Executing things that need to be execute"
	show_pourcent_to_ihm "$POURCENT"
	show_first_info_to_ihm "Dernieres verifications"
	show_second_info_to_ihm ""
	$STATIC_EXE_PLACE/busybox sh /tmp/update/things_to_execute.sh
fi

POURCENT=100
show_pourcent_to_ihm "$POURCENT"
show_first_info_to_ihm "Fin de la mise a jour"
show_second_info_to_ihm ""

printit "Update finished at $($STATIC_EXE_PLACE/busybox date +%H:%M:%S) the $($STATIC_EXE_PLACE/busybox date +%d/%m/%Y )"
PS_TO_KILL=$($STATIC_EXE_PLACE/busybox fuser /dev/watchdog)
#PS_TO_KILL=$($STATIC_EXE_PLACE/busybox ps |grep busybox |grep watchdog |awk '{print $1}')
if [ 'ri'"$PS_TO_KILL"'en' = "rien" ]
then
	ask_very_hard_reboot
	exit
else
	$STATIC_EXE_PLACE/busybox kill -9 $PS_TO_KILL
fi